// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using Hyak.Common;
using Microsoft.WindowsAzure.Management.RemoteApp;
using Microsoft.WindowsAzure.Management.RemoteApp.Models;
using Newtonsoft.Json.Linq;

namespace Microsoft.WindowsAzure.Management.RemoteApp
{
    /// <summary>
    /// RemoteApp virtual network operations.
    /// </summary>
    internal partial class VNetOperations : IServiceOperations<RemoteAppManagementClient>, IVNetOperations
    {
        /// <summary>
        /// Initializes a new instance of the VNetOperations class.
        /// </summary>
        /// <param name='client'>
        /// Reference to the service client.
        /// </param>
        internal VNetOperations(RemoteAppManagementClient client)
        {
            this._client = client;
        }
        
        private RemoteAppManagementClient _client;
        
        /// <summary>
        /// Gets a reference to the
        /// Microsoft.WindowsAzure.Management.RemoteApp.RemoteAppManagementClient.
        /// </summary>
        public RemoteAppManagementClient Client
        {
            get { return this._client; }
        }
        
        /// <summary>
        /// Creates or updates a virtual network.
        /// </summary>
        /// <param name='vNetName'>
        /// Required. RemoteApp virtual network name.
        /// </param>
        /// <param name='vNetDetails'>
        /// Required. Details about the virtual network to create or update.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response containing the operation tracking id.
        /// </returns>
        public async Task<OperationResultWithTrackingId> CreateOrUpdateAsync(string vNetName, VNetParameter vNetDetails, CancellationToken cancellationToken)
        {
            // Validate
            if (vNetName == null)
            {
                throw new ArgumentNullException("vNetName");
            }
            if (vNetDetails == null)
            {
                throw new ArgumentNullException("vNetDetails");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("vNetName", vNetName);
                tracingParameters.Add("vNetDetails", vNetDetails);
                TracingAdapter.Enter(invocationId, this, "CreateOrUpdateAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/services/";
            if (this.Client.RdfeNamespace != null)
            {
                url = url + Uri.EscapeDataString(this.Client.RdfeNamespace);
            }
            url = url + "/vnets/";
            url = url + Uri.EscapeDataString(vNetName);
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2014-09-01");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Put;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2014-08-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                JObject vNetParameterValue = new JObject();
                requestDoc = vNetParameterValue;
                
                if (vNetDetails.Region != null)
                {
                    vNetParameterValue["Region"] = vNetDetails.Region;
                }
                
                if (vNetDetails.VnetAddressSpaces != null)
                {
                    JArray vnetAddressSpacesArray = new JArray();
                    foreach (string vnetAddressSpacesItem in vNetDetails.VnetAddressSpaces)
                    {
                        vnetAddressSpacesArray.Add(vnetAddressSpacesItem);
                    }
                    vNetParameterValue["VnetAddressSpaces"] = vnetAddressSpacesArray;
                }
                
                if (vNetDetails.GatewaySubnet != null)
                {
                    vNetParameterValue["GatewaySubnet"] = vNetDetails.GatewaySubnet;
                }
                
                if (vNetDetails.LocalAddressSpaces != null)
                {
                    JArray localAddressSpacesArray = new JArray();
                    foreach (string localAddressSpacesItem in vNetDetails.LocalAddressSpaces)
                    {
                        localAddressSpacesArray.Add(localAddressSpacesItem);
                    }
                    vNetParameterValue["LocalAddressSpaces"] = localAddressSpacesArray;
                }
                
                if (vNetDetails.VpnAddress != null)
                {
                    vNetParameterValue["VpnAddress"] = vNetDetails.VpnAddress;
                }
                
                if (vNetDetails.DnsServers != null)
                {
                    JArray dnsServersArray = new JArray();
                    foreach (string dnsServersItem in vNetDetails.DnsServers)
                    {
                        dnsServersArray.Add(dnsServersItem);
                    }
                    vNetParameterValue["DnsServers"] = dnsServersArray;
                }
                
                if (vNetDetails.GatewayIp != null)
                {
                    vNetParameterValue["GatewayIp"] = vNetDetails.GatewayIp;
                }
                
                vNetParameterValue["GatewayType"] = ((int)vNetDetails.GatewayType);
                
                requestContent = requestDoc.ToString(Newtonsoft.Json.Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = MediaTypeHeaderValue.Parse("application/json; charset=utf-8");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    OperationResultWithTrackingId result = null;
                    // Deserialize Response
                    result = new OperationResultWithTrackingId();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-remoteapp-operation-tracking-id"))
                    {
                        result.TrackingId = httpResponse.Headers.GetValues("x-remoteapp-operation-tracking-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Deletes the specified RemoteApp virtual network
        /// </summary>
        /// <param name='vNetName'>
        /// Required. RemoteApp virtual network name.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response containing the operation tracking id.
        /// </returns>
        public async Task<OperationResultWithTrackingId> DeleteAsync(string vNetName, CancellationToken cancellationToken)
        {
            // Validate
            if (vNetName == null)
            {
                throw new ArgumentNullException("vNetName");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("vNetName", vNetName);
                TracingAdapter.Enter(invocationId, this, "DeleteAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/services/";
            if (this.Client.RdfeNamespace != null)
            {
                url = url + Uri.EscapeDataString(this.Client.RdfeNamespace);
            }
            url = url + "/vnets/";
            url = url + Uri.EscapeDataString(vNetName);
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2014-09-01");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Delete;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2014-08-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    OperationResultWithTrackingId result = null;
                    // Deserialize Response
                    result = new OperationResultWithTrackingId();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-remoteapp-operation-tracking-id"))
                    {
                        result.TrackingId = httpResponse.Headers.GetValues("x-remoteapp-operation-tracking-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Gets the specified virtual network's configuration.
        /// </summary>
        /// <param name='vNetName'>
        /// Required. RemoteApp virtual network name.
        /// </param>
        /// <param name='includeSharedKey'>
        /// Required. Set to true to get back the shared key used to configure
        /// the VPN connection
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// RemoteApp virtual network information.
        /// </returns>
        public async Task<VNetResult> GetAsync(string vNetName, bool includeSharedKey, CancellationToken cancellationToken)
        {
            // Validate
            if (vNetName == null)
            {
                throw new ArgumentNullException("vNetName");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("vNetName", vNetName);
                tracingParameters.Add("includeSharedKey", includeSharedKey);
                TracingAdapter.Enter(invocationId, this, "GetAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/services/";
            if (this.Client.RdfeNamespace != null)
            {
                url = url + Uri.EscapeDataString(this.Client.RdfeNamespace);
            }
            url = url + "/vnets/";
            url = url + Uri.EscapeDataString(vNetName);
            List<string> queryParameters = new List<string>();
            queryParameters.Add("includeKey=" + Uri.EscapeDataString(includeSharedKey.ToString().ToLower()));
            queryParameters.Add("api-version=2014-09-01");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept", "application/json; charset=utf-8");
                httpRequest.Headers.Add("x-ms-version", "2014-08-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    VNetResult result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new VNetResult();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            VNet vNetInstance = new VNet();
                            result.VNet = vNetInstance;
                            
                            JToken nameValue = responseDoc["Name"];
                            if (nameValue != null && nameValue.Type != JTokenType.Null)
                            {
                                string nameInstance = ((string)nameValue);
                                vNetInstance.Name = nameInstance;
                            }
                            
                            JToken stateValue = responseDoc["State"];
                            if (stateValue != null && stateValue.Type != JTokenType.Null)
                            {
                                VNetState stateInstance = ((VNetState)(((int)stateValue)));
                                vNetInstance.State = stateInstance;
                            }
                            
                            JToken stateInfoValue = responseDoc["StateInfo"];
                            if (stateInfoValue != null && stateInfoValue.Type != JTokenType.Null)
                            {
                                string stateInfoInstance = ((string)stateInfoValue);
                                vNetInstance.StateInfo = stateInfoInstance;
                            }
                            
                            JToken sharedKeyValue = responseDoc["SharedKey"];
                            if (sharedKeyValue != null && sharedKeyValue.Type != JTokenType.Null)
                            {
                                string sharedKeyInstance = ((string)sharedKeyValue);
                                vNetInstance.SharedKey = sharedKeyInstance;
                            }
                            
                            JToken upgradeInProgressValue = responseDoc["UpgradeInProgress"];
                            if (upgradeInProgressValue != null && upgradeInProgressValue.Type != JTokenType.Null)
                            {
                                bool upgradeInProgressInstance = ((bool)upgradeInProgressValue);
                                vNetInstance.UpgradeInProgress = upgradeInProgressInstance;
                            }
                            
                            JToken regionValue = responseDoc["Region"];
                            if (regionValue != null && regionValue.Type != JTokenType.Null)
                            {
                                string regionInstance = ((string)regionValue);
                                vNetInstance.Region = regionInstance;
                            }
                            
                            JToken vnetAddressSpacesArray = responseDoc["VnetAddressSpaces"];
                            if (vnetAddressSpacesArray != null && vnetAddressSpacesArray.Type != JTokenType.Null)
                            {
                                foreach (JToken vnetAddressSpacesValue in ((JArray)vnetAddressSpacesArray))
                                {
                                    vNetInstance.VnetAddressSpaces.Add(((string)vnetAddressSpacesValue));
                                }
                            }
                            
                            JToken gatewaySubnetValue = responseDoc["GatewaySubnet"];
                            if (gatewaySubnetValue != null && gatewaySubnetValue.Type != JTokenType.Null)
                            {
                                string gatewaySubnetInstance = ((string)gatewaySubnetValue);
                                vNetInstance.GatewaySubnet = gatewaySubnetInstance;
                            }
                            
                            JToken localAddressSpacesArray = responseDoc["LocalAddressSpaces"];
                            if (localAddressSpacesArray != null && localAddressSpacesArray.Type != JTokenType.Null)
                            {
                                foreach (JToken localAddressSpacesValue in ((JArray)localAddressSpacesArray))
                                {
                                    vNetInstance.LocalAddressSpaces.Add(((string)localAddressSpacesValue));
                                }
                            }
                            
                            JToken vpnAddressValue = responseDoc["VpnAddress"];
                            if (vpnAddressValue != null && vpnAddressValue.Type != JTokenType.Null)
                            {
                                string vpnAddressInstance = ((string)vpnAddressValue);
                                vNetInstance.VpnAddress = vpnAddressInstance;
                            }
                            
                            JToken dnsServersArray = responseDoc["DnsServers"];
                            if (dnsServersArray != null && dnsServersArray.Type != JTokenType.Null)
                            {
                                foreach (JToken dnsServersValue in ((JArray)dnsServersArray))
                                {
                                    vNetInstance.DnsServers.Add(((string)dnsServersValue));
                                }
                            }
                            
                            JToken gatewayIpValue = responseDoc["GatewayIp"];
                            if (gatewayIpValue != null && gatewayIpValue.Type != JTokenType.Null)
                            {
                                string gatewayIpInstance = ((string)gatewayIpValue);
                                vNetInstance.GatewayIp = gatewayIpInstance;
                            }
                            
                            JToken gatewayTypeValue = responseDoc["GatewayType"];
                            if (gatewayTypeValue != null && gatewayTypeValue.Type != JTokenType.Null)
                            {
                                GatewayType gatewayTypeInstance = ((GatewayType)(((int)gatewayTypeValue)));
                                vNetInstance.GatewayType = gatewayTypeInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Gets the status of the call from ResetVpnSharedKey.
        /// </summary>
        /// <param name='trackingId'>
        /// Required. Tracking ID returned by ResetVpnSharedKey
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Current state of a long running operation.
        /// </returns>
        public async Task<VNetOperationStatusResult> GetResetVpnSharedKeyOperationStatusAsync(string trackingId, CancellationToken cancellationToken)
        {
            // Validate
            if (trackingId == null)
            {
                throw new ArgumentNullException("trackingId");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("trackingId", trackingId);
                TracingAdapter.Enter(invocationId, this, "GetResetVpnSharedKeyOperationStatusAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/services/";
            if (this.Client.RdfeNamespace != null)
            {
                url = url + Uri.EscapeDataString(this.Client.RdfeNamespace);
            }
            url = url + "/vnets/operationStatus/";
            url = url + Uri.EscapeDataString(trackingId);
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2014-09-01");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept", "application/json; charset=utf-8");
                httpRequest.Headers.Add("x-ms-version", "2014-08-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    VNetOperationStatusResult result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new VNetOperationStatusResult();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            JToken statusValue = responseDoc["Status"];
                            if (statusValue != null && statusValue.Type != JTokenType.Null)
                            {
                                VNetOperationStatus statusInstance = ((VNetOperationStatus)(((int)statusValue)));
                                result.Status = statusInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Gets a configuration script to configure the VPN deviceto connect
        /// to the given virtual network. Run this script on the VPN device.
        /// </summary>
        /// <param name='vNetName'>
        /// Required. RemoteApp virtual network name.
        /// </param>
        /// <param name='vendor'>
        /// Optional. Vendor
        /// </param>
        /// <param name='vpnDevice'>
        /// Optional. Model of the VPN
        /// </param>
        /// <param name='osFamily'>
        /// Optional. OS Family
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Script that can be used to set up a VPN device.
        /// </returns>
        public async Task<VNetConfigScriptResult> GetVpnDeviceConfigScriptAsync(string vNetName, string vendor, string vpnDevice, string osFamily, CancellationToken cancellationToken)
        {
            // Validate
            if (vNetName == null)
            {
                throw new ArgumentNullException("vNetName");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("vNetName", vNetName);
                tracingParameters.Add("vendor", vendor);
                tracingParameters.Add("vpnDevice", vpnDevice);
                tracingParameters.Add("osFamily", osFamily);
                TracingAdapter.Enter(invocationId, this, "GetVpnDeviceConfigScriptAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/services/";
            if (this.Client.RdfeNamespace != null)
            {
                url = url + Uri.EscapeDataString(this.Client.RdfeNamespace);
            }
            url = url + "/vnets/";
            url = url + Uri.EscapeDataString(vNetName);
            url = url + "/vpnscript";
            List<string> queryParameters = new List<string>();
            if (vendor != null)
            {
                queryParameters.Add("vendor=" + Uri.EscapeDataString(vendor));
            }
            if (vpnDevice != null)
            {
                queryParameters.Add("platform=" + Uri.EscapeDataString(vpnDevice));
            }
            if (osFamily != null)
            {
                queryParameters.Add("osFamily=" + Uri.EscapeDataString(osFamily));
            }
            queryParameters.Add("api-version=2014-09-01");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept", "application/json; charset=utf-8");
                httpRequest.Headers.Add("x-ms-version", "2014-08-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    VNetConfigScriptResult result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new VNetConfigScriptResult();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            string configScriptInstance = ((string)responseDoc);
                            result.ConfigScript = configScriptInstance;
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Gets details of supported VPN devices
        /// </summary>
        /// <param name='vNetName'>
        /// Required. RemoteApp virtual network name.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The VPN device information.
        /// </returns>
        public async Task<VNetVpnDeviceResult> GetVpnDevicesAsync(string vNetName, CancellationToken cancellationToken)
        {
            // Validate
            if (vNetName == null)
            {
                throw new ArgumentNullException("vNetName");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("vNetName", vNetName);
                TracingAdapter.Enter(invocationId, this, "GetVpnDevicesAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/services/";
            if (this.Client.RdfeNamespace != null)
            {
                url = url + Uri.EscapeDataString(this.Client.RdfeNamespace);
            }
            url = url + "/vnets/";
            url = url + Uri.EscapeDataString(vNetName);
            url = url + "/vpndevices";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2014-09-01");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept", "application/json; charset=utf-8");
                httpRequest.Headers.Add("x-ms-version", "2014-08-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    VNetVpnDeviceResult result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new VNetVpnDeviceResult();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            JToken vendorsArray = responseDoc["Vendors"];
                            if (vendorsArray != null && vendorsArray.Type != JTokenType.Null)
                            {
                                foreach (JToken vendorsValue in ((JArray)vendorsArray))
                                {
                                    Vendor vendorInstance = new Vendor();
                                    result.Vendors.Add(vendorInstance);
                                    
                                    JToken nameValue = vendorsValue["name"];
                                    if (nameValue != null && nameValue.Type != JTokenType.Null)
                                    {
                                        string nameInstance = ((string)nameValue);
                                        vendorInstance.Name = nameInstance;
                                    }
                                    
                                    JToken platformsArray = vendorsValue["Platforms"];
                                    if (platformsArray != null && platformsArray.Type != JTokenType.Null)
                                    {
                                        foreach (JToken platformsValue in ((JArray)platformsArray))
                                        {
                                            Platform platformInstance = new Platform();
                                            vendorInstance.Platforms.Add(platformInstance);
                                            
                                            JToken nameValue2 = platformsValue["name"];
                                            if (nameValue2 != null && nameValue2.Type != JTokenType.Null)
                                            {
                                                string nameInstance2 = ((string)nameValue2);
                                                platformInstance.Name = nameInstance2;
                                            }
                                            
                                            JToken osFamiliesArray = platformsValue["OsFamilies"];
                                            if (osFamiliesArray != null && osFamiliesArray.Type != JTokenType.Null)
                                            {
                                                foreach (JToken osFamiliesValue in ((JArray)osFamiliesArray))
                                                {
                                                    OsFamily osFamilyInstance = new OsFamily();
                                                    platformInstance.OsFamilies.Add(osFamilyInstance);
                                                    
                                                    JToken nameValue3 = osFamiliesValue["name"];
                                                    if (nameValue3 != null && nameValue3.Type != JTokenType.Null)
                                                    {
                                                        string nameInstance3 = ((string)nameValue3);
                                                        osFamilyInstance.Name = nameInstance3;
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Gets a list of virtual network configurations.
        /// </summary>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// List of virtual networks.
        /// </returns>
        public async Task<VNetListResult> ListAsync(CancellationToken cancellationToken)
        {
            // Validate
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                TracingAdapter.Enter(invocationId, this, "ListAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/services/";
            if (this.Client.RdfeNamespace != null)
            {
                url = url + Uri.EscapeDataString(this.Client.RdfeNamespace);
            }
            url = url + "/vnets";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2014-09-01");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept", "application/json; charset=utf-8");
                httpRequest.Headers.Add("x-ms-version", "2014-08-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    VNetListResult result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new VNetListResult();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            JToken vNetListArray = responseDoc;
                            if (vNetListArray != null && vNetListArray.Type != JTokenType.Null)
                            {
                                foreach (JToken vNetListValue in ((JArray)vNetListArray))
                                {
                                    VNet vNetInstance = new VNet();
                                    result.VNetList.Add(vNetInstance);
                                    
                                    JToken nameValue = vNetListValue["Name"];
                                    if (nameValue != null && nameValue.Type != JTokenType.Null)
                                    {
                                        string nameInstance = ((string)nameValue);
                                        vNetInstance.Name = nameInstance;
                                    }
                                    
                                    JToken stateValue = vNetListValue["State"];
                                    if (stateValue != null && stateValue.Type != JTokenType.Null)
                                    {
                                        VNetState stateInstance = ((VNetState)(((int)stateValue)));
                                        vNetInstance.State = stateInstance;
                                    }
                                    
                                    JToken stateInfoValue = vNetListValue["StateInfo"];
                                    if (stateInfoValue != null && stateInfoValue.Type != JTokenType.Null)
                                    {
                                        string stateInfoInstance = ((string)stateInfoValue);
                                        vNetInstance.StateInfo = stateInfoInstance;
                                    }
                                    
                                    JToken sharedKeyValue = vNetListValue["SharedKey"];
                                    if (sharedKeyValue != null && sharedKeyValue.Type != JTokenType.Null)
                                    {
                                        string sharedKeyInstance = ((string)sharedKeyValue);
                                        vNetInstance.SharedKey = sharedKeyInstance;
                                    }
                                    
                                    JToken upgradeInProgressValue = vNetListValue["UpgradeInProgress"];
                                    if (upgradeInProgressValue != null && upgradeInProgressValue.Type != JTokenType.Null)
                                    {
                                        bool upgradeInProgressInstance = ((bool)upgradeInProgressValue);
                                        vNetInstance.UpgradeInProgress = upgradeInProgressInstance;
                                    }
                                    
                                    JToken regionValue = vNetListValue["Region"];
                                    if (regionValue != null && regionValue.Type != JTokenType.Null)
                                    {
                                        string regionInstance = ((string)regionValue);
                                        vNetInstance.Region = regionInstance;
                                    }
                                    
                                    JToken vnetAddressSpacesArray = vNetListValue["VnetAddressSpaces"];
                                    if (vnetAddressSpacesArray != null && vnetAddressSpacesArray.Type != JTokenType.Null)
                                    {
                                        foreach (JToken vnetAddressSpacesValue in ((JArray)vnetAddressSpacesArray))
                                        {
                                            vNetInstance.VnetAddressSpaces.Add(((string)vnetAddressSpacesValue));
                                        }
                                    }
                                    
                                    JToken gatewaySubnetValue = vNetListValue["GatewaySubnet"];
                                    if (gatewaySubnetValue != null && gatewaySubnetValue.Type != JTokenType.Null)
                                    {
                                        string gatewaySubnetInstance = ((string)gatewaySubnetValue);
                                        vNetInstance.GatewaySubnet = gatewaySubnetInstance;
                                    }
                                    
                                    JToken localAddressSpacesArray = vNetListValue["LocalAddressSpaces"];
                                    if (localAddressSpacesArray != null && localAddressSpacesArray.Type != JTokenType.Null)
                                    {
                                        foreach (JToken localAddressSpacesValue in ((JArray)localAddressSpacesArray))
                                        {
                                            vNetInstance.LocalAddressSpaces.Add(((string)localAddressSpacesValue));
                                        }
                                    }
                                    
                                    JToken vpnAddressValue = vNetListValue["VpnAddress"];
                                    if (vpnAddressValue != null && vpnAddressValue.Type != JTokenType.Null)
                                    {
                                        string vpnAddressInstance = ((string)vpnAddressValue);
                                        vNetInstance.VpnAddress = vpnAddressInstance;
                                    }
                                    
                                    JToken dnsServersArray = vNetListValue["DnsServers"];
                                    if (dnsServersArray != null && dnsServersArray.Type != JTokenType.Null)
                                    {
                                        foreach (JToken dnsServersValue in ((JArray)dnsServersArray))
                                        {
                                            vNetInstance.DnsServers.Add(((string)dnsServersValue));
                                        }
                                    }
                                    
                                    JToken gatewayIpValue = vNetListValue["GatewayIp"];
                                    if (gatewayIpValue != null && gatewayIpValue.Type != JTokenType.Null)
                                    {
                                        string gatewayIpInstance = ((string)gatewayIpValue);
                                        vNetInstance.GatewayIp = gatewayIpInstance;
                                    }
                                    
                                    JToken gatewayTypeValue = vNetListValue["GatewayType"];
                                    if (gatewayTypeValue != null && gatewayTypeValue.Type != JTokenType.Null)
                                    {
                                        GatewayType gatewayTypeInstance = ((GatewayType)(((int)gatewayTypeValue)));
                                        vNetInstance.GatewayType = gatewayTypeInstance;
                                    }
                                }
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Posts a request to asynchronously generate a new shared key for the
        /// specified virtual network and returns a tracking ID for the
        /// operation.Use GetResetVpnSharedKeyOperationStatus with this
        /// tracking ID to determine when the operation has completed.  When
        /// the status returns Success,call Get passing the virtual network
        /// name and true for the includeSharedKey parameter. This will return
        /// a VNetResult containing the new key.Call GetVpnConfigScript to get
        /// back the configuration script to configure the VPN device. Run
        /// this script using the new key on the VPN device.
        /// </summary>
        /// <param name='vNetName'>
        /// Required. RemoteApp virtual network name.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response containing the operation tracking id.
        /// </returns>
        public async Task<OperationResultWithTrackingId> ResetVpnSharedKeyAsync(string vNetName, CancellationToken cancellationToken)
        {
            // Validate
            if (vNetName == null)
            {
                throw new ArgumentNullException("vNetName");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("vNetName", vNetName);
                TracingAdapter.Enter(invocationId, this, "ResetVpnSharedKeyAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/services/";
            if (this.Client.RdfeNamespace != null)
            {
                url = url + Uri.EscapeDataString(this.Client.RdfeNamespace);
            }
            url = url + "/vnets/";
            url = url + Uri.EscapeDataString(vNetName);
            url = url + "/resetsharedkey";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2014-09-01");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept", "application/json; charset=utf-8");
                httpRequest.Headers.Add("x-ms-version", "2014-08-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    OperationResultWithTrackingId result = null;
                    // Deserialize Response
                    result = new OperationResultWithTrackingId();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-remoteapp-operation-tracking-id"))
                    {
                        result.TrackingId = httpResponse.Headers.GetValues("x-remoteapp-operation-tracking-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
    }
}
